@@ -22,9 +22,9 @@ from collections import defaultdict
 
 import z3c.form
 import z3c.form.browser.text
+import zc.table.table
 from z3c.form import form, field, button, widget
 from z3c.form.interfaces import DISPLAY_MODE, HIDDEN_MODE, NO_VALUE
-from zc.table import table
 from zc.table.column import GetterColumn
 
 from zope.browserpage.viewpagetemplatefile import ViewPageTemplateFile
@@ -49,32 +49,26 @@ from zope.traversing.browser.absoluteurl import absoluteURL
 from zope.traversing.browser.interfaces import IAbsoluteURL
 
 from schooltool.app.interfaces import ISchoolToolApplication
+from schooltool.app.browser.app import EditRelationships
+from schooltool.app.browser.app import RelationshipAddTableMixin
+from schooltool.app.browser.app import RelationshipRemoveTableMixin
 from schooltool.basicperson.browser.demographics import (
     DemographicsView,
     FlourishDemographicsView, FlourishReorderDemographicsView)
 from schooltool.basicperson.interfaces import IAddEditViewTitle
 from schooltool.basicperson.interfaces import ILimitKeysLabel
 from schooltool.basicperson.interfaces import ILimitKeysHint
-from schooltool.common.inlinept import InheritTemplate
+from schooltool.common.inlinept import InheritTemplate, InlineViewPageTemplate
 from schooltool.resource.interfaces import IBookingCalendar
 from schooltool.resource.interfaces import (
              IResourceContainer, IResourceTypeInformation, IResourceSubTypes,
              IResource, IEquipment, ILocation, IResourceDemographicsFields)
 from schooltool.resource.resource import Resource, Location, Equipment
-from schooltool.table.interfaces import IFilterWidget
-from schooltool.table.interfaces import ITableFormatter
-from schooltool.table.table import url_cell_formatter
-from schooltool.table.table import CheckboxColumn
-from schooltool.table.table import FilterWidget
-from schooltool.table.table import SchoolToolTableFormatter
 from schooltool.person.browser.person import PersonFilterWidget
 from schooltool.resource.interfaces import IResourceFactoryUtility
 from schooltool.schoolyear.interfaces import ISchoolYearContainer
-from schooltool.skin.flourish.containers import TableContainerView
-from schooltool.skin.flourish.page import RefineLinksViewlet, Page
-from schooltool.skin.flourish.page import Content
-from schooltool.skin.flourish.page import ModalFormLinkViewlet
-from schooltool.skin.flourish.form import DialogForm, AddForm
+from schooltool.skin import flourish
+from schooltool import table
 
 from schooltool.common import SchoolToolMessage as _
 
@@ -102,8 +96,9 @@ class ResourceContainerView(oldform.FormBase):
     def __init__(self, context, request):
         oldform.FormBase.__init__(self, context, request)
         self.resourceType = self.request.get('SEARCH_TYPE','|').split('|')[0]
-        self.filter_widget = queryMultiAdapter((self.getResourceUtility(),
-                                                self.request), IFilterWidget)
+        self.filter_widget = queryMultiAdapter(
+            (self.getResourceUtility(), self.request),
+            table.interfaces.IFilterWidget)
 
 
     def getSubTypes(self):
@@ -165,12 +160,13 @@ class ResourceContainerView(oldform.FormBase):
         return self.filter_widget.filter(values)
 
     def renderResourceTable(self):
-        columns = [CheckboxColumn(prefix="delete", name='delete', title=u'')]
+        columns = [table.column.CheckboxColumn(
+                       prefix="delete", name='delete', title=u'')]
         available_columns = self.columns()
-        available_columns[0].cell_formatter = url_cell_formatter
+        available_columns[0].cell_formatter = table.table.url_cell_formatter
 
         columns.extend(available_columns)
-        formatter = table.StandaloneFullFormatter(
+        formatter = zc.table.table.StandaloneFullFormatter(
             self.context, self.request, self.filter(self.context.values()),
             columns=columns,
             sort_on=self.sortOn(),
@@ -179,22 +175,83 @@ class ResourceContainerView(oldform.FormBase):
         return formatter()
 
 
-class FlourishResourceContainerView(TableContainerView):
+class FlourishResourcesView(flourish.page.Page):
 
-    @property
-    def done_link(self):
-        app = ISchoolToolApplication(None)
-        return absoluteURL(app, self.request) + '/manage'
+    content_template = InlineViewPageTemplate('''
+      <div tal:content="structure context/schooltool:content/ajax/table" />
+    ''')
 
-    def getColumnsAfter(self):
+
+class ResourcesTable(table.ajax.Table):
+
+    def columns(self):
+        default = table.ajax.Table.columns(self)
         description = GetterColumn(
             name='description',
             title=_(u'Description'),
             getter=lambda i, f: i.description or '')
-        return [description]
+        return default + [description]
 
 
-class BaseTypeFilter(FilterWidget):
+class ResourceListTable(ResourcesTable):
+
+    @property
+    def source(self):
+        return ISchoolToolApplication(None)['resources']
+
+    def items(self):
+        return self.context
+
+
+class LocationListTable(ResourceListTable):
+
+    def items(self):
+        return [r for r in self.context
+                if ILocation(r, None) is not None]
+
+
+class EquipmentListTable(ResourceListTable):
+
+    def items(self):
+        return [r for r in self.context
+                if IEquipment(r, None) is not None]
+
+
+class EditResourceRelationships(EditRelationships):
+
+    def getAvailableItemsContainer(self):
+        return ISchoolToolApplication(None)['resources']
+
+
+class EditLocationRelationships(EditResourceRelationships):
+    pass
+
+
+class EditEquipmentRelationships(EditResourceRelationships):
+    pass
+
+
+class LocationAddRelationshipTable(RelationshipAddTableMixin,
+                                   ResourcesTable):
+    pass
+
+
+class LocationRemoveRelationshipTable(RelationshipRemoveTableMixin,
+                                      ResourcesTable):
+    pass
+
+
+class EquipmentAddRelationshipTable(RelationshipAddTableMixin,
+                                    ResourcesTable):
+    pass
+
+
+class EquipmentRemoveRelationshipTable(RelationshipRemoveTableMixin,
+                                       ResourcesTable):
+    pass
+
+
+class BaseTypeFilter(table.table.FilterWidget):
     """Base Type Filter"""
 
     def render(self):
@@ -333,7 +390,8 @@ class ResourceContainerFilterWidget(PersonFilterWidget):
             type = type.split('|')[0]
             utility = queryUtility(IResourceFactoryUtility,
                                    name=type, default=None)
-            filter_widget = queryMultiAdapter((utility,self.request), IFilterWidget)
+            filter_widget = queryMultiAdapter((utility,self.request),
+                                              table.interfaces.IFilterWidget)
             if filter_widget:
                 results = filter_widget.filter(results)
 
@@ -344,6 +402,9 @@ class FlourishResourceContainerFilterWidget(ResourceContainerFilterWidget):
 
     template = ViewPageTemplateFile('templates/f_resource_filter.pt')
 
+    search_title_id = 'SEARCH_TITLE'
+    search_type_id = 'SEARCH_TYPE'
+
     def types(self):
         options = [
             {'id': 'equipment',
@@ -356,13 +417,13 @@ class FlourishResourceContainerFilterWidget(ResourceContainerFilterWidget):
         return options
 
     def filter(self, results):
-        if 'SEARCH_TITLE' in self.request:
-            searchstr = self.request['SEARCH_TITLE'].lower()
+        if self.search_title_id in self.request:
+            searchstr = self.request[self.search_title_id].lower()
             results = [item for item in results
                        if searchstr in item.title.lower() or
                        (item.description and searchstr in item.description.lower())]
-        if 'SEARCH_TYPE' in self.request:
-            type = self.request['SEARCH_TYPE']
+        if self.search_type_id in self.request:
+            type = self.request[self.search_type_id]
             if not type:
                 return results
             results = [resource for resource in results
@@ -370,11 +431,30 @@ class FlourishResourceContainerFilterWidget(ResourceContainerFilterWidget):
         return results
 
 
-class ResourceContainerLinks(RefineLinksViewlet):
+class ResourcesTableFilter(table.ajax.TableFilter,
+                           FlourishResourceContainerFilterWidget):
+
+    template = ViewPageTemplateFile('templates/f_resource_table_filter.pt')
+
+    @property
+    def search_title_id(self):
+        return self.manager.html_id+"-title"
+
+    @property
+    def search_group_id(self):
+        return self.manager.html_id+"-group"
+
+    def filter(self, results):
+        if self.ignoreRequest:
+            return results
+        return FlourishResourceContainerFilterWidget.filter(self, results)
+
+
+class ResourceContainerLinks(flourish.page.RefineLinksViewlet):
     """Resource container links viewlet."""
 
 
-class ResourceImportLinks(RefineLinksViewlet):
+class ResourceImportLinks(flourish.page.RefineLinksViewlet):
     """Resource import links viewlet."""
 
 
@@ -493,7 +573,7 @@ class BaseResourceView(form.Form, ResourceFieldGenerator):
                 self.widgets[widget].mode = HIDDEN_MODE
 
 
-class FlourishBaseResourceView(Page, BaseResourceView):
+class FlourishBaseResourceView(flourish.page.Page, BaseResourceView):
 
     def update(self):
         BaseResourceView.update(self)
@@ -502,16 +582,6 @@ class FlourishBaseResourceView(Page, BaseResourceView):
     def canModify(self):
         return canAccess(self.context.__parent__, '__delitem__')
 
-    @property
-    def leaders_table(self):
-        return self.getTable(list(self.context.leaders))
-
-    def getTable(self, items):
-        persons = ISchoolToolApplication(None)['persons']
-        result = getMultiAdapter((persons, self.request), ITableFormatter)
-        result.setUp(table_formatter=table.StandaloneFullFormatter, items=items)
-        return result
-
     def has_leaders(self):
         return bool(list(self.context.leaders))
 
@@ -654,9 +724,9 @@ class BaseResourceEditView(ErrorMessageBase, form.EditForm,
         self.actions['cancel'].addClass('button-cancel')
 
 
-class BaseFlourishResourceAddForm(AddForm):
+class BaseFlourishResourceAddForm(flourish.form.AddForm):
 
-    template = InheritTemplate(Page.template)
+    template = InheritTemplate(flourish.page.Page.template)
     label = None
 
     def createAndAdd(self, data):
@@ -671,7 +741,8 @@ class BaseFlourishResourceAddForm(AddForm):
         return resource
 
     def nextURL(self):
-        return absoluteURL(self._resource or self.context, self.request)
+        return absoluteURL(getattr(self, '_resource', None) or self.context,
+                           self.request)
 
 
 ###############  Resource add/edit views ################
@@ -703,7 +774,7 @@ class ResourceEditView(BaseResourceForm, BaseResourceEditView):
     label = _('Edit resource')
 
 
-class FlourishResourceEditView(Page, ResourceEditView):
+class FlourishResourceEditView(flourish.page.Page, ResourceEditView):
 
     label = None
     demo_legend = _('Resource attributes')
@@ -742,7 +813,7 @@ class LocationEditView(BaseLocationForm, BaseResourceEditView):
     label = _('Edit location')
 
 
-class FlourishLocationEditView(Page, LocationEditView):
+class FlourishLocationEditView(flourish.page.Page, LocationEditView):
 
     label = None
     demo_legend = _('Location attributes')
@@ -781,7 +852,7 @@ class EquipmentEditView(BaseEquipmentForm, BaseResourceEditView):
     label = _('Edit equipment')
 
 
-class FlourishEquipmentEditView(Page, EquipmentEditView):
+class FlourishEquipmentEditView(flourish.page.Page, EquipmentEditView):
 
     label = None
     demo_legend = _('Equipment attributes')
@@ -790,7 +861,7 @@ class FlourishEquipmentEditView(Page, EquipmentEditView):
         EquipmentEditView.update(self)
 
 
-class FlourishResourceContainerTableFormatter(SchoolToolTableFormatter):
+class FlourishResourceContainerTableFormatter(table.table.SchoolToolTableFormatter):
 
     # XXX: hack to customize the table class
     def render(self):
@@ -804,14 +875,14 @@ class FlourishResourceContainerTableFormatter(SchoolToolTableFormatter):
         return formatter()
 
 
-class ResourceLinks(RefineLinksViewlet):
+class ResourceLinks(flourish.page.RefineLinksViewlet):
 
     @property
     def title(self):
         return self.context.title
 
 
-class ResourceActions(RefineLinksViewlet): pass
+class ResourceActions(flourish.page.RefineLinksViewlet): pass
 
 
 class FlourishBookResourceView(BrowserView):
@@ -824,7 +895,7 @@ class FlourishBookResourceView(BrowserView):
         self.request.response.redirect(url)
 
 
-class FlourishResourceDeleteView(DialogForm, form.Form):
+class FlourishResourceDeleteView(flourish.form.DialogForm, form.Form):
 
     dialog_submit_actions = ('delete',)
     dialog_close_actions = ('cancel',)
@@ -847,7 +918,7 @@ class FlourishResourceDeleteView(DialogForm, form.Form):
         self.actions['cancel'].addClass('button-cancel')
 
 
-class FlourishResourceDeleteLink(ModalFormLinkViewlet):
+class FlourishResourceDeleteLink(flourish.page.ModalFormLinkViewlet):
 
     @property
     def dialog_title(self):
@@ -856,7 +927,7 @@ class FlourishResourceDeleteLink(ModalFormLinkViewlet):
         return translate(title, context=self.request)
 
 
-class FlourishManageResourcesOverview(Content):
+class FlourishManageResourcesOverview(flourish.page.Content):
 
     body_template = ViewPageTemplateFile(
         'templates/f_manage_resources_overview.pt')
