@@ -104,7 +104,7 @@ def doctest_IndexedGetterColumn():
     Indexed column requires an additional keyword argument (index)
     for it's constructor:
 
-        >>> from schooltool.table.catalog import IndexedGetterColumn
+        >>> from schooltool.table.column import IndexedGetterColumn
         >>> column = IndexedGetterColumn(index='title', getter=lambda i, f: i.title)
         >>> column.index
         'title'
@@ -168,7 +168,7 @@ def doctest_IndexedLocaleAwareGetterColumn():
     Indexed column requires an additional keyword argument (index)
     for it's constructor:
 
-        >>> from schooltool.table.catalog import IndexedLocaleAwareGetterColumn
+        >>> from schooltool.table.column import IndexedLocaleAwareGetterColumn
         >>> column = IndexedLocaleAwareGetterColumn(index='title',
         ...                                         getter=lambda i, f: i.title)
         >>> column.index
@@ -207,7 +207,7 @@ def doctest_IndexedTableFormatter_columns():
         >>> formatter = IndexedTableFormatter(None, None)
         >>> columns = formatter.columns()
         >>> columns
-        [<schooltool.table.catalog.IndexedGetterColumn object at ...>]
+        [<schooltool.table.column.IndexedGetterColumn object at ...>]
 
         >>> columns[0].title
         u'Title'
